@@ -65,7 +65,8 @@ int o2fsck_test_inode_allocated(o2fsck_state *ost, uint64_t blkno)
 struct dirblock_data {
 	o2fsck_state 	*ost;
 	ocfs2_filesys 	*fs;
-	char 		*buf;
+	char 		*dirblock_buf;
+	char 		*inoblock_buf;
 	errcode_t	ret;
 	o2fsck_strings	strings;
 	uint64_t	last_ino;
@@ -105,9 +106,10 @@ static errcode_t fix_dirent_dots(o2fsck_state *ost, o2fsck_dirblock_entry *dbe,
 	errcode_t ret = 0;
 
 	if (!expect_dots) {
-	       	if (!dirent_has_dots(dirent, 1) && !dirent_has_dots(dirent, 2))
+		if (!dirent->inode ||
+		    (!dirent_has_dots(dirent, 1) && !dirent_has_dots(dirent, 2)))
 			goto out;
-			return 0;
+
 		if (prompt(ost, PY, PR_DIRENT_DOTTY_DUP,
 			   "Duplicate '%.*s' directory entry found, remove "
 			   "it?", dirent->name_len, dirent->name)) {
@@ -128,6 +130,7 @@ static errcode_t fix_dirent_dots(o2fsck_state *ost, o2fsck_dirblock_entry *dbe,
 
 		dirent->name_len = expect_dots;
 		memset(dirent->name, '.', expect_dots);
+		dirent->file_type = OCFS2_FT_DIR;
 		changed_len = 1;
 		*flags |= OCFS2_DIRENT_CHANGED;
 	}
@@ -183,16 +186,49 @@ out:
 	return ret;
 }
 
-/* we just copy ext2's fixing behaviour here.  'left' is the number of bytes
- * from the start of the dirent struct to the end of the block. */
+static int dirent_leaves_partial(struct ocfs2_dir_entry *dirent, int left)
+{
+	left -= dirent->rec_len;
+	return (left > 0 && left < OCFS2_DIR_MEMBER_LEN);
+}
+
+/*
+ * The caller has found that either of rec_len or name_len are garbage.  The
+ * caller trusts us to fix them up in place and will be checking them again
+ * before proceeding.  We have to update the lengths to make forward progress.
+ * 'left' is the number of bytes from the start of this dirent struct that
+ * remain in the block.  
+ *
+ * We're called for invalid dirents, and having a dirent
+ * that leaves a partial dirent at the end of the block is considered invalid,
+ * and we pad out partials at the end of this call so we can't be called here
+ * with left < OCFS2_DIR_MEMBER_LEN.
+ *
+ * we're pretty limited in the repairs we can make:
+ *
+ * - We can't just set name_len if rec_len looks valid, we might guess 
+ *   name_len wrong and create a bogus file name.
+ * - we can't just set rec_len based on name_len.  rec_len could have
+ *   included an arbitrary part of the name from a previously freed dirent.
+ */
 static void fix_dirent_lengths(struct ocfs2_dir_entry *dirent,
 			       int left, struct ocfs2_dir_entry *prev,
 			       int *flags)
 {
-	/* special casing an empty dirent that doesn't include the
-	 * extra rec_len alignment */
-	if ((left >= OCFS2_DIR_MEMBER_LEN) && 
-			(dirent->rec_len == OCFS2_DIR_MEMBER_LEN)) {
+	/* 
+	 * as described above we can't reconstruct either value if it is
+	 * complete nonsense.  We can only proceed if we can work off of
+	 * one that is kind of valid looking.
+	 * name_len could well be 0 from the dirent being cleared.
+	 */
+	if (dirent->rec_len < OCFS2_DIR_MEMBER_LEN ||
+	   (dirent->rec_len > left ||
+	    dirent->name_len > left))
+	    goto wipe;
+
+	/* if we see a dirent with no file name then we remove it by
+	 * shifting the remaining dirents forward */
+	if ((dirent->rec_len == OCFS2_DIR_MEMBER_LEN)) {
 		char *cp = (char *)dirent;
 		left -= dirent->rec_len;
 		memmove(cp, cp + dirent->rec_len, left);
@@ -200,28 +236,51 @@ static void fix_dirent_lengths(struct ocfs2_dir_entry *dirent,
 		goto out;
 	}
 
-	/* clamp rec_len to the remainder of the block if name_len
-	 * is within the block */
-	if (dirent->rec_len > left && dirent->name_len <= left) {
-		dirent->rec_len = left;
+	/* if rec_len just appears to be mis-rounded in a way that doesn't
+	 * affect following dirents then we can probably save this dirent */
+	if (OCFS2_DIR_REC_LEN(dirent->name_len) != dirent->rec_len &&
+	    OCFS2_DIR_REC_LEN(dirent->name_len) == 
+					OCFS2_DIR_REC_LEN(dirent->rec_len)) {
+		dirent->rec_len = OCFS2_DIR_REC_LEN(dirent->name_len);
+		left -= dirent->rec_len;
 		goto out;
 	}
 
-	/* from here on in we're losing directory entries by adding their
-	 * space onto previous entries.  If we think we can trust this 
-	 * entry's length we leave potential later entries intact.  
-	 * otherwise we consume all the space left in the block */
-	if (prev && ((dirent->rec_len & OCFS2_DIR_ROUND) == 0) &&
-		(dirent->rec_len <= left)) {
-		prev->rec_len += left;
-	} else {
-		dirent->rec_len = left;
+	/* if name_len is too far off, however, we're going to lose this
+	 * dirent.. we might be able to just lose this one dirent if rec_len
+	 * appears to be intact. */
+	if ((dirent->rec_len & OCFS2_DIR_ROUND) == 0 &&
+	    !dirent_leaves_partial(dirent, left)) {
+		left -= dirent->rec_len;
 		dirent->name_len = 0;
 		dirent->inode = 0;
 		dirent->file_type = OCFS2_FT_UNKNOWN;
+		goto out;
 	}
 
+	/* 
+	 * if we can't trust rec_len, however, then we don't know where the
+	 * next dirent might begin.  We've lost the trail of dirents created by
+	 * the file system and run the risk of parsing file names as dirents.
+	 * So we're forced to wipe the block and leave the rest to lost+found.
+	 */
+wipe:
+	dirent->rec_len = left;
+	dirent->name_len = 0;
+	dirent->inode = 0;
+	dirent->file_type = OCFS2_FT_UNKNOWN;
+	left = 0;
 out:
+
+	/* 
+	 * rec_len must be valid and left must reflect the space *after* the
+	 * current dirent by this point.  if there isn't enough room for
+	 * another dirent after the one we've just repaired then we tack the
+	 * remaining space onto the current dirent.
+	 */
+	if (dirent_leaves_partial(dirent, left))
+		dirent->rec_len += left;
+
 	*flags |= OCFS2_DIRENT_CHANGED;
 }
 
@@ -272,7 +331,7 @@ static void fix_dirent_inode(o2fsck_state *ost, o2fsck_dirblock_entry *dbe,
 		goto out;
 	}
 
-	if (!o2fsck_test_inode_allocated(ost, dbe->e_ino) &&
+	if (!o2fsck_test_inode_allocated(ost, dirent->inode) &&
 	    prompt(ost, PY, PR_DIRENT_INODE_FREE,
 		   "Directory entry '%.*s' refers to inode number "
 		   "%"PRIu64" which isn't allocated, clear the entry?", 
@@ -525,14 +584,18 @@ out:
 	return ret;
 }
 
-static int corrupt_dirent(struct ocfs2_dir_entry *dirent, int left)
+static int corrupt_dirent_lengths(struct ocfs2_dir_entry *dirent, int left)
 {
 	if ((dirent->rec_len >= OCFS2_DIR_REC_LEN(1)) &&
 	    ((dirent->rec_len & OCFS2_DIR_ROUND) == 0) &&
 	    (dirent->rec_len <= left) &&
-	    (OCFS2_DIR_REC_LEN(dirent->name_len) <= dirent->rec_len))
+	    (OCFS2_DIR_REC_LEN(dirent->name_len) <= dirent->rec_len) &&
+	    !dirent_leaves_partial(dirent, left))
 		return 0;
 
+	verbosef("corrupt dirent: %"PRIu64" rec_len %u name_len %u\n",
+		dirent->inode, dirent->rec_len, dirent->name_len);
+
 	return 1;
 }
 
@@ -543,35 +606,53 @@ static unsigned pass2_dir_block_iterate(o2fsck_dirblock_entry *dbe,
 	struct dirblock_data *dd = priv_data;
 	struct ocfs2_dir_entry *dirent, *prev = NULL;
 	unsigned int offset = 0, ret_flags = 0;
-	errcode_t ret;
+	struct ocfs2_dinode *di = (struct ocfs2_dinode *)dd->inoblock_buf; 
+	errcode_t ret = 0;
 
 	if (!o2fsck_test_inode_allocated(dd->ost, dbe->e_ino)) {
 		printf("Directory block %"PRIu64" belongs to directory inode "
 		       "%"PRIu64" which isn't allocated.  Ignoring this "
 		       "block.", dbe->e_blkno, dbe->e_ino);
-		return 0;
+		goto out;
 	}
 
 	if (dbe->e_ino != dd->last_ino) {
 		o2fsck_strings_free(&dd->strings);
 		dd->last_ino = dbe->e_ino;
+
+		ret = ocfs2_read_inode(dd->ost->ost_fs, dbe->e_ino,
+				       dd->inoblock_buf);
+		if (ret) {
+			com_err(whoami, ret, "while reading dir inode %"PRIu64,
+				dbe->e_ino);
+			ret_flags |= OCFS2_DIRENT_ABORT;
+			goto out;
+		}
+
+		verbosef("dir inode %"PRIu64" i_size %"PRIu64"\n",
+			 dbe->e_ino, di->i_size);
+
 	}
 
- 	ret = ocfs2_read_dir_block(dd->fs, dbe->e_blkno, dd->buf);
+	verbosef("dir block %"PRIu64" block offs %"PRIu64" in ino\n",
+		 dbe->e_blkno, dbe->e_blkcount);
+
+	if (dbe->e_blkcount >= ocfs2_blocks_in_bytes(dd->fs, di->i_size))
+		goto out;
+
+ 	ret = ocfs2_read_dir_block(dd->fs, dbe->e_blkno, dd->dirblock_buf);
 	if (ret && ret != OCFS2_ET_DIR_CORRUPTED) {
 		com_err(whoami, ret, "while reading dir block %"PRIu64,
 			dbe->e_blkno);
 		goto out;
 	}
 
-	verbosef("dir block %"PRIu64"\n", dbe->e_blkno);
-
 	while (offset < dd->fs->fs_blocksize) {
-		dirent = (struct ocfs2_dir_entry *)(dd->buf + offset);
+		dirent = (struct ocfs2_dir_entry *)(dd->dirblock_buf + offset);
 
-		verbosef("checking dirent offset %d, ino %"PRIu64" rec_len "
-			"%"PRIu16" name_len %"PRIu8" file_type %"PRIu8"\n",
-			offset, dirent->inode, dirent->rec_len, 
+		verbosef("checking dirent offset %d, rec_len %"PRIu16" "
+			 "name_len %"PRIu8" file_type %"PRIu8"\n",
+			offset, dirent->rec_len, 
 			dirent->name_len, dirent->file_type);
 
 		/* XXX I wonder if we should be checking that the padding
@@ -579,7 +660,8 @@ static unsigned pass2_dir_block_iterate(o2fsck_dirblock_entry *dbe,
 
 		/* if we can't trust this dirent then fix it up or skip
 		 * the whole block */
-		if (corrupt_dirent(dirent, dd->fs->fs_blocksize - offset)) {
+		if (corrupt_dirent_lengths(dirent,
+					   dd->fs->fs_blocksize - offset)) {
 			if (!prompt(dd->ost, PY, PR_DIRENT_LENGTH,
 				    "Directory inode %"PRIu64" "
 				    "corrupted in logical block %"PRIu64" "
@@ -653,7 +735,8 @@ next:
 	}
 
 	if (ret_flags & OCFS2_DIRENT_CHANGED) {
-		ret = ocfs2_write_dir_block(dd->fs, dbe->e_blkno, dd->buf);
+		ret = ocfs2_write_dir_block(dd->fs, dbe->e_blkno,
+					    dd->dirblock_buf);
 		if (ret) {
 			com_err(whoami, ret, "while writing dir block %"PRIu64,
 				dbe->e_blkno);
@@ -669,8 +752,8 @@ out:
 
 errcode_t o2fsck_pass2(o2fsck_state *ost)
 {
-	errcode_t retval;
 	o2fsck_dir_parent *dp;
+	errcode_t ret;
 	struct dirblock_data dd = {
 		.ost = ost,
 		.fs = ost->ost_fs,
@@ -681,9 +764,19 @@ errcode_t o2fsck_pass2(o2fsck_state *ost)
 
 	o2fsck_strings_init(&dd.strings);
 
-	retval = ocfs2_malloc_block(ost->ost_fs->fs_io, &dd.buf);
-	if (retval)
-		return retval;
+	ret = ocfs2_malloc_block(ost->ost_fs->fs_io, &dd.dirblock_buf);
+	if (ret) {
+		com_err(whoami, ret, "while allocating a block buffer to "
+			"store directory blocks.");
+		goto out;
+	}
+
+	ret = ocfs2_malloc_block(ost->ost_fs->fs_io, &dd.inoblock_buf);
+	if (ret) {
+		com_err(whoami, ret, "while allocating a block buffer to "
+			"store a directory inode.");
+		goto out;
+	}
 
 	/* 
 	 * Mark the root directory's dirent parent as itself if we found the
@@ -704,6 +797,10 @@ errcode_t o2fsck_pass2(o2fsck_state *ost)
 	o2fsck_dir_block_iterate(&ost->ost_dirblocks, pass2_dir_block_iterate, 
 			 	 &dd);
 	o2fsck_strings_free(&dd.strings);
-	ocfs2_free(&dd.buf);
-	return 0;
+out:
+	if (dd.dirblock_buf)
+		ocfs2_free(&dd.dirblock_buf);
+	if (dd.inoblock_buf)
+		ocfs2_free(&dd.inoblock_buf);
+	return ret;
 }
