@@ -71,7 +71,7 @@ out:
 	return ret;
 }
 
-static errcode_t ocfs2_read_super(ocfs2_filesys *fs, int superblock)
+errcode_t ocfs2_read_super(ocfs2_filesys *fs, uint64_t superblock, char *sb)
 {
 	errcode_t ret;
 	char *blk;
@@ -92,7 +92,13 @@ static errcode_t ocfs2_read_super(ocfs2_filesys *fs, int superblock)
 		goto out_blk;
 
 	ocfs2_swap_inode_to_cpu(di);
-	fs->fs_super = di;
+
+	if (!sb)
+		fs->fs_super = di;
+	else {
+		memcpy(sb, blk, fs->fs_blocksize);
+		ocfs2_free(&blk);
+	}
 
 	return 0;
 
@@ -129,6 +135,48 @@ out_blk:
 	return ret;
 }
 
+errcode_t ocfs2_write_backup_super(ocfs2_filesys *fs, uint64_t blkno)
+{
+	errcode_t ret;
+	char *buf = NULL;
+	struct ocfs2_dinode *di;
+
+	if (!(fs->fs_flags & OCFS2_FLAG_RW))
+		return OCFS2_ET_RO_FILESYS;
+
+	ret = ocfs2_malloc_block(fs->fs_io, &buf);
+	if (ret)
+		goto out_blk;
+
+	memcpy(buf, (char *)fs->fs_super, fs->fs_blocksize);
+	di = (struct ocfs2_dinode *)buf;
+
+	ret = OCFS2_ET_BAD_MAGIC;
+	if (memcmp(di->i_signature, OCFS2_SUPER_BLOCK_SIGNATURE,
+		   strlen(OCFS2_SUPER_BLOCK_SIGNATURE)))
+		goto out_blk;
+
+	di->i_blkno = blkno;
+	OCFS2_SET_COMPAT_FEATURE(OCFS2_RAW_SB(di),
+				 OCFS2_FEATURE_COMPAT_BACKUP_SB);
+	ret = ocfs2_write_inode(fs, blkno, buf);
+	if (ret)
+		goto out_blk;
+
+	ret = 0;
+
+out_blk:
+	if (buf)
+		ocfs2_free(&buf);
+	return ret;
+}
+
+int ocfs2_mount_local(ocfs2_filesys *fs)
+{
+	return OCFS2_RAW_SB(fs->fs_super)->s_feature_incompat &
+	       OCFS2_FEATURE_INCOMPAT_LOCAL_MOUNT;
+}
+
 errcode_t ocfs2_open(const char *name, int flags,
 		     unsigned int superblock, unsigned int block_size,
 		     ocfs2_filesys **ret_fs)
@@ -173,18 +221,19 @@ errcode_t ocfs2_open(const char *name, int flags,
 		if (!block_size)
 			goto out;
 		io_set_blksize(fs->fs_io, block_size);
-		ret = ocfs2_read_super(fs, superblock);
+		ret = ocfs2_read_super(fs, (uint64_t)superblock, NULL);
 	} else {
 		superblock = OCFS2_SUPER_BLOCK_BLKNO;
 		if (block_size) {
 			io_set_blksize(fs->fs_io, block_size);
-			ret = ocfs2_read_super(fs, superblock);
+			ret = ocfs2_read_super(fs, (uint64_t)superblock, NULL);
 		} else {
 			for (block_size = io_get_blksize(fs->fs_io);
 			     block_size <= OCFS2_MAX_BLOCKSIZE;
 			     block_size <<= 1) {
 				io_set_blksize(fs->fs_io, block_size);
-				ret = ocfs2_read_super(fs, superblock);
+				ret = ocfs2_read_super(fs, (uint64_t)superblock,
+						       NULL);
 				if (ret == OCFS2_ET_BAD_MAGIC)
 					continue;
 				break;
@@ -210,6 +259,13 @@ errcode_t ocfs2_open(const char *name, int flags,
 		goto out;
 #endif
 
+	if (flags & OCFS2_FLAG_STRICT_COMPAT_CHECK) {
+		ret = OCFS2_ET_UNSUPP_FEATURE;
+		if (OCFS2_RAW_SB(fs->fs_super)->s_feature_compat &
+		    ~OCFS2_LIB_FEATURE_COMPAT_SUPP)
+			    goto out;
+	}
+
 	ret = OCFS2_ET_UNSUPP_FEATURE;
 	if (OCFS2_RAW_SB(fs->fs_super)->s_feature_incompat &
 	    ~OCFS2_LIB_FEATURE_INCOMPAT_SUPP)
