@@ -62,6 +62,11 @@
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
 
+#define MOUNT_LOCAL             1
+#define MOUNT_CLUSTER           2
+#define MOUNT_LOCAL_STR         "local"
+#define MOUNT_CLUSTER_STR       "cluster"
+
 #define MIN_RESERVED_TAIL_BLOCKS    8
 
 #define SUPERBLOCK_BLOCKS       3
@@ -85,7 +90,6 @@
 #define ONE_MB_SHIFT           20
 #define ONE_GB_SHIFT           30
 
-#define BITMAP_WARNING_LEN     1572864
 #define BITMAP_AUTO_MAX        786432
 
 #define AUTO_CLUSTERSIZE       65536
@@ -101,6 +105,11 @@ enum {
 	SFI_OTHER
 };
 
+enum ocfs2_fs_types {
+	FS_DEFAULT,
+	FS_DATAFILES,
+	FS_MAIL
+};
 
 typedef struct _SystemFileInfo SystemFileInfo;
 
@@ -193,6 +202,8 @@ struct _State {
 	int force;
 	int prompt;
 	int hb_dev;
+	int mount;
+	int no_backup_super;
 
 	uint32_t blocksize;
 	uint32_t blocksize_bits;
@@ -229,6 +240,8 @@ struct _State {
 	uint16_t tail_group_bits;
 	uint32_t first_cluster_group;
 	uint64_t first_cluster_group_blkno;
+
+	enum ocfs2_fs_types fs_type;
 };
 
 int ocfs2_check_volume(State *s);
